@@ -18,6 +18,7 @@ import (
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/featureflags"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/tls"
 	"gitlab.com/gitlab-org/gitlab-runner/session"
+	"gitlab.com/gitlab-org/gitlab-runner/session/proxy"
 	"gitlab.com/gitlab-org/gitlab-runner/session/terminal"
 )
 
@@ -335,6 +336,10 @@ func (b *Build) run(ctx context.Context, executor Executor) (err error) {
 		b.Session.SetInteractiveTerminal(term)
 	}
 
+	if proxyPooler, ok := executor.(proxy.Pooler); b.Session != nil && ok {
+		b.Session.SetProxyPool(proxyPooler)
+	}
+
 	// Run build script
 	go func() {
 		buildFinish <- b.executeScript(runContext, executor)
@@ -681,12 +686,6 @@ func (b *Build) GetRemoteURL() string {
 	return fmt.Sprintf("%sgitlab-ci-token:%s@%s/%s.git", splits[0], ciJobToken, splits[1], ciProjectPath)
 }
 
-// GetGitDepth is deprecated and will be removed in 12.0, use build.GitInfo.Depth instead
-// TODO: Remove in 12.0
-func (b *Build) GetGitDepth() string {
-	return b.GetAllVariables().Get("GIT_DEPTH")
-}
-
 func (b *Build) GetGitStrategy() GitStrategy {
 	switch b.GetAllVariables().Get("GIT_STRATEGY") {
 	case "clone":
@@ -815,10 +814,6 @@ func (b *Build) Duration() time.Duration {
 	return time.Since(b.createdAt)
 }
 
-func (b *Build) RefspecsAvailable() bool {
-	return len(b.GitInfo.Refspecs) > 0
-}
-
 func NewBuild(jobData JobResponse, runnerConfig *RunnerConfig, systemInterrupt chan os.Signal, executorData ExecutorData) (*Build, error) {
 	// Attempt to perform a deep copy of the RunnerConfig
 	runnerConfigCopy, err := runnerConfig.DeepCopy()
