@@ -17,29 +17,42 @@ func TestVariablesMasking(t *testing.T) {
 		"containing",
 	}
 
-	buffer := New()
+	buffer, err := New()
+	require.NoError(t, err)
+	defer buffer.Close()
+
 	buffer.SetMasked(maskedValues)
 
-	_, err := buffer.Write([]byte(traceMessage))
+	_, err = buffer.Write([]byte(traceMessage))
 	require.NoError(t, err)
 
-	err = buffer.Close()
+	buffer.Finish()
+
+	content, err := buffer.Bytes(0, 1000)
 	require.NoError(t, err)
 
-	assert.Equal(t, "Th[MASKED] [MASKED] the [MASKED] message [MASKED] [MASKED] [MASKED]s", buffer.String())
+	assert.Equal(t, "Th[MASKED] [MASKED] the [MASKED] message [MASKED] [MASKED] [MASKED]s", string(content))
 }
 
 func TestTraceLimit(t *testing.T) {
 	traceMessage := "This is the long message"
 
-	buffer := New()
+	buffer, err := New()
+	require.NoError(t, err)
+	defer buffer.Close()
+
 	buffer.SetLimit(10)
 
-	_, err := buffer.Write([]byte(traceMessage))
+	for i := 0; i < 100; i++ {
+		_, err = buffer.Write([]byte(traceMessage))
+		require.NoError(t, err)
+	}
+
+	buffer.Finish()
+
+	content, err := buffer.Bytes(0, 1000)
 	require.NoError(t, err)
 
-	err = buffer.Close()
-	require.NoError(t, err)
-
-	assert.Contains(t, buffer.String(), "Job's log exceeded limit of")
+	assert.Equal(t, "This is the\n\x1b[31;1mJob's log exceeded limit of 10 bytes.\x1b[0;m\n",
+		string(content))
 }
